@@ -1,23 +1,26 @@
 # -*- coding: utf-8 -*-
 
-from gi.repository import GObject, Peas, Gtk, Gdk # pylint: disable-msg=E0611
-from gi.repository import Gio, Pango, Totem # pylint: disable-msg=E0611
-
-import xmlrpclib
+import xmlrpc.client
 import threading
-import xdg.BaseDirectory
-from os import sep, path, mkdir
+import zlib
+from os import sep
 import gettext
 
-from hash import hash_file
+import gi
+gi.require_version('Peas', '1.0')
+gi.require_version('Gtk', '3.0')
+gi.require_version('Totem', '1.0')
+from gi.repository import GLib, GObject # pylint: disable=wrong-import-position
+from gi.repository import Peas, Gtk, Gdk # pylint: disable=wrong-import-position,
+from gi.repository import Gio, Pango, Totem # pylint: disable=wrong-import-position,no-name-in-module
+
+from hash import hash_file # pylint: disable=wrong-import-position
 
 gettext.textdomain ("totem")
 
 D_ = gettext.dgettext
 _ = gettext.gettext
 
-GObject.threads_init ()
-
 USER_AGENT = 'Totem'
 OK200 = '200 OK'
 TOTEM_REMOTE_COMMAND_REPLACE = 14
@@ -26,10 +29,12 @@ SUBTITLES_EXT = [
     "asc",
     "txt",
     "sub",
+    "vtt",
     "srt",
     "smi",
     "ssa",
     "ass",
+    "mpl",
 ]
 
 # Map of the language codes used by opensubtitles.org's API to their
@@ -38,6 +43,7 @@ LANGUAGES_STR = [ (D_('iso_639_3', 'Albanian'), 'sq'),
          (D_('iso_639_3', 'Arabic'), 'ar'),
          (D_('iso_639_3', 'Armenian'), 'hy'),
          (D_('iso_639_3', 'Neo-Aramaic, Assyrian'), 'ay'),
+         (D_('iso_639_3', 'Basque'), 'eu'),
          (D_('iso_639_3', 'Bosnian'), 'bs'),
          (_('Brazilian Portuguese'), 'pb'),
          (D_('iso_639_3', 'Bulgarian'), 'bg'),
@@ -103,6 +109,7 @@ LANGUAGES = {'sq':'alb',
          'nl':'dut',
          'en':'eng',
          'eo':'epo',
+         'eu':'eus',
          'et':'est',
          'fi':'fin',
          'fr':'fre',
@@ -141,6 +148,11 @@ LANGUAGES = {'sq':'alb',
          'uk':'ukr',
          'vi':'vie',}
 
+def _cache_subtitles_dir ():
+    bpath = GLib.get_user_cache_dir() + sep
+    bpath += 'totem' + sep + 'subtitles' + sep
+    return bpath
+
 class SearchThread (threading.Thread):
     """
     This is the thread started when the dialog is searching for subtitles
@@ -174,7 +186,7 @@ class SearchThread (threading.Thread):
         return results
 
     def get_message (self):
-        message = ''
+        message = _(u'Searching for subtitles…')
 
         self._lock.acquire (True)
         if self._done:
@@ -222,7 +234,7 @@ class DownloadThread (threading.Thread):
         return subtitles
 
     def get_message (self):
-        message = ''
+        message = _(u'Downloading the subtitles…')
 
         self._lock.acquire (True)
         if self._done:
@@ -241,7 +253,7 @@ class DownloadThread (threading.Thread):
 
 # OpenSubtitles.org API abstraction
 
-class OpenSubtitlesModel (object):
+class OpenSubtitlesModel:
     """
     This contains the logic of the opensubtitles service.
     """
@@ -250,10 +262,10 @@ class OpenSubtitlesModel (object):
         self._token = None
 
         try:
-            import locale
-            (language_code, _encoding) = locale.getlocale ()
+            import locale # pylint: disable=import-outside-toplevel
+            (language_code, _) = locale.getlocale ()
             self.lang = LANGUAGES[language_code.split ('_')[0]]
-        except (ImportError, IndexError, AttributeError):
+        except (ImportError, IndexError, AttributeError, KeyError):
             self.lang = 'eng'
 
         self._lock = threading.Lock ()
@@ -271,7 +283,7 @@ class OpenSubtitlesModel (object):
             # We have already logged-in before, check the connection
             try:
                 result = self._server.NoOperation (self._token)
-            except (xmlrpclib.Fault, xmlrpclib.ProtocolError):
+            except (xmlrpc.client.Fault, xmlrpc.client.ProtocolError):
                 pass
             if result and result['status'] != OK200:
                 return (True, '')
@@ -279,7 +291,7 @@ class OpenSubtitlesModel (object):
         try:
             result = self._server.LogIn (username, password, self.lang,
                                          USER_AGENT)
-        except (xmlrpclib.Fault, xmlrpclib.ProtocolError):
+        except (xmlrpc.client.Fault, xmlrpc.client.ProtocolError):
             pass
 
         if result and result.get ('status') == OK200:
@@ -319,14 +331,14 @@ class OpenSubtitlesModel (object):
             try:
                 result = self._server.SearchSubtitles (self._token,
                                                        [searchdata])
-            except xmlrpclib.ProtocolError:
-                message = _(u'Could not contact the OpenSubtitles website')
+            except xmlrpc.client.ProtocolError:
+                message = _(u'Could not contact the OpenSubtitles website.')
 
             if result.get ('data'):
                 self._lock.release ()
                 return (result['data'], message)
-            else:
-                message = _(u'No results found')
+
+            message = _(u'No results found.')
         else:
             message = log_in_message
 
@@ -338,15 +350,16 @@ class OpenSubtitlesModel (object):
         self._lock.acquire (True)
 
         message = ''
-        error_message = _(u'Could not contact the OpenSubtitles website')
+        error_message = _(u'Could not contact the OpenSubtitles website.')
 
         (log_in_success, log_in_message) = self._log_in ()
 
         if log_in_success:
+            result = None
             try:
                 result = self._server.DownloadSubtitles (self._token,
                                                          [subtitle_id])
-            except xmlrpclib.ProtocolError:
+            except xmlrpc.client.ProtocolError:
                 message = error_message
 
             if result and result.get ('status') == OK200:
@@ -356,14 +369,11 @@ class OpenSubtitlesModel (object):
                     self._lock.release ()
                     return (None, error_message)
 
-                import StringIO, gzip, base64
-                subtitle_decoded = base64.decodestring (subtitle64)
-                subtitle_gzipped = StringIO.StringIO (subtitle_decoded)
-                subtitle_gzipped_file = gzip.GzipFile (fileobj=subtitle_gzipped)
+                subtitle_unzipped = zlib.decompress(GLib.base64_decode (subtitle64), 47)
 
                 self._lock.release ()
 
-                return (subtitle_gzipped_file.read (), message)
+                return (subtitle_unzipped, message)
         else:
             message = log_in_message
 
@@ -371,10 +381,11 @@ class OpenSubtitlesModel (object):
 
         return (None, message)
 
-class OpenSubtitles (GObject.Object, Peas.Activatable):
+class OpenSubtitles (GObject.Object, # pylint: disable=R0902
+                     Peas.Activatable):
     __gtype_name__ = 'OpenSubtitles'
 
-    object = GObject.property (type = GObject.Object)
+    object = GObject.Property (type = GObject.Object)
 
     def __init__ (self):
         GObject.Object.__init__ (self)
@@ -384,9 +395,6 @@ class OpenSubtitles (GObject.Object, Peas.Activatable):
         schema = 'org.gnome.totem.plugins.opensubtitles'
         self._settings = Gio.Settings.new (schema)
 
-        self._manager = None
-        self._menu_id = None
-        self._action_group = None
         self._action = None
 
         self._find_button = None
@@ -414,16 +422,19 @@ class OpenSubtitles (GObject.Object, Peas.Activatable):
         # are related to.
         self._filename = None
 
-        self._manager = self._totem.get_ui_manager ()
         self._append_menu ()
 
-        self._totem.connect ('file-opened', self.__on_totem__file_opened)
-        self._totem.connect ('file-closed', self.__on_totem__file_closed)
+        self._totem.connect ('file-opened', self.__on_totem__file_opened) # pylint: disable=no-member
+        self._totem.connect ('file-closed', self.__on_totem__file_closed) # pylint: disable=no-member
 
         # Obtain the ServerProxy and init the model
-        server = xmlrpclib.Server ('http://api.opensubtitles.org/xml-rpc')
+        server = xmlrpc.client.Server ('http://api.opensubtitles.org/xml-rpc')
         self._model = OpenSubtitlesModel (server)
 
+        mrl = self._totem.get_current_mrl ()
+        if mrl:
+            self.__on_totem__file_opened(self._totem, self._totem.get_current_mrl ())
+
     def do_deactivate (self):
         if self._dialog:
             self._dialog.destroy ()
@@ -436,8 +447,8 @@ class OpenSubtitles (GObject.Object, Peas.Activatable):
     def _build_dialog (self):
         builder = Totem.plugin_load_interface ("opensubtitles",
                                                "opensubtitles.ui", True,
-                                               self._totem.get_main_window (),
-                                               self)
+                                               self._totem.get_main_window (), # pylint: disable=no-member
+                                               None)
 
         # Obtain all the widgets we need to initialize
         combobox = builder.get_object ('language_combobox')
@@ -473,7 +484,6 @@ class OpenSubtitles (GObject.Object, Peas.Activatable):
         # Set up the results treeview
         renderer = Gtk.CellRendererText ()
         self._tree_view.set_model (self._list_store)
-        self._tree_view.set_headers_visible (False)
         renderer.set_property ('ellipsize', Pango.EllipsizeMode.END)
         column = Gtk.TreeViewColumn (_(u"Subtitles"), renderer, text=0)
         column.set_resizable (True)
@@ -499,8 +509,7 @@ class OpenSubtitles (GObject.Object, Peas.Activatable):
         # Set up signals
 
         combobox.connect ('changed', self.__on_combobox__changed)
-        self._dialog.connect ('delete-event', self._dialog.hide_on_delete)
-        self._dialog.set_transient_for (self._totem.get_main_window ())
+        self._dialog.set_transient_for (self._totem.get_main_window ()) # pylint: disable=no-member
         self._dialog.set_position (Gtk.WindowPosition.CENTER_ON_PARENT)
 
         # Connect the callbacks
@@ -511,7 +520,7 @@ class OpenSubtitles (GObject.Object, Peas.Activatable):
         self._tree_view.connect ('row-activated',
                                self.__on_treeview__row_activate)
 
-    def _show_dialog (self, _action):
+    def _show_dialog (self, *_):
         if not self._dialog:
             self._build_dialog ()
 
@@ -520,43 +529,24 @@ class OpenSubtitles (GObject.Object, Peas.Activatable):
         self._progress.set_fraction (0.0)
 
     def _append_menu (self):
-        self._action_group = Gtk.ActionGroup (name='OpenSubtitles')
-
-        tooltip_text = _(u"Download movie subtitles from OpenSubtitles")
-        self._action = Gtk.Action (name='opensubtitles',
-                                 label=_(u'_Download Movie Subtitles…'),
-                                 tooltip=tooltip_text,
-                                 stock_id=None)
-
-        self._action_group.add_action (self._action)
-
-        self._manager.insert_action_group (self._action_group, 0)
-
-        self._menu_id = self._manager.new_merge_id ()
-        merge_path = '/tmw-menubar/view/subtitles/subtitle-download-placeholder'
-        self._manager.add_ui (self._menu_id,
-                             merge_path,
-                             'opensubtitles',
-                             'opensubtitles',
-                             Gtk.UIManagerItemType.MENUITEM,
-                             False
-                            )
-        self._action.set_visible (True)
-
-        self._manager.ensure_update ()
-
+        self._action = Gio.SimpleAction.new ("opensubtitles", None)
         self._action.connect ('activate', self._show_dialog)
+        self._totem.add_action (self._action) # pylint: disable=no-member
+        self._totem.set_accels_for_action ("app.opensubtitles",  # pylint: disable=no-member
+                                           ["<Primary><Shift>s"])
 
-        self._action.set_sensitive (self._totem.is_playing () and
+        menu = self._totem.get_menu_section ("subtitle-download-placeholder") # pylint: disable=no-member
+        menu.append (_(u'_Download Movie Subtitles…'), "app.opensubtitles")
+
+        self._action.set_enabled (self._totem.is_playing () and # pylint: disable=no-member
                   self._check_allowed_scheme () and
                                   not self._check_is_audio ())
 
     def _check_allowed_scheme (self):
-        current_file = Gio.file_new_for_uri (self._totem.get_current_mrl ())
+        current_file = Gio.file_new_for_uri (self._totem.get_current_mrl ()) # pylint: disable=no-member
         scheme = current_file.get_uri_scheme ()
 
-        if (scheme == 'dvd' or scheme == 'http' or
-            scheme == 'dvb' or scheme == 'vcd'):
+        if scheme in ('dvd', 'http', 'dvb', 'vcd'):
             return False
 
         return True
@@ -565,18 +555,16 @@ class OpenSubtitles (GObject.Object, Peas.Activatable):
         # FIXME need to use something else here
         # I think we must use video widget metadata but I don't found a way
         # to get this info from python
-        filename = self._totem.get_current_mrl ()
+        filename = self._totem.get_current_mrl () # pylint: disable=no-member
         if Gio.content_type_guess (filename, '')[0].split ('/')[0] == 'audio':
             return True
         return False
 
     def _delete_menu (self):
-        self._manager.remove_action_group (self._action_group)
-        self._manager.remove_ui (self._menu_id)
+        self._totem.empty_menu_section ("subtitle-download-placeholder") # pylint: disable=no-member
 
     def _get_results (self, movie_hash, movie_size):
         self._list_store.clear ()
-        self._tree_view.set_headers_visible (False)
         self._apply_button.set_sensitive (False)
         self._find_button.set_sensitive (False)
 
@@ -585,10 +573,10 @@ class OpenSubtitles (GObject.Object, Peas.Activatable):
 
         thread = SearchThread (self._model, movie_hash, movie_size)
         thread.start ()
-        GObject.idle_add (self._populate_treeview, thread)
+        GLib.idle_add (self._populate_treeview, thread)
 
         self._progress.set_text (_(u'Searching subtitles…'))
-        GObject.timeout_add (350, self._progress_bar_increment, thread)
+        GLib.timeout_add (350, self._progress_bar_increment, thread)
 
     def _populate_treeview (self, search_thread):
         if not search_thread.done:
@@ -596,7 +584,6 @@ class OpenSubtitles (GObject.Object, Peas.Activatable):
 
         results = search_thread.get_results ()
         if results:
-            self._apply_button.set_sensitive (True)
             for sub_data in results:
                 if not SUBTITLES_EXT.count (sub_data['SubFormat']):
                     continue
@@ -604,15 +591,12 @@ class OpenSubtitles (GObject.Object, Peas.Activatable):
                                         sub_data['SubFormat'],
                                         sub_data['SubRating'],
                                         sub_data['IDSubtitleFile'],])
-                self._tree_view.set_headers_visible (True)
-        else:
-            self._apply_button.set_sensitive (False)
 
         self._dialog.get_window ().set_cursor (None)
 
         return False
 
-    def _save_selected_subtitle (self, filename=None):
+    def _save_selected_subtitle (self):
         cursor = Gdk.Cursor.new (Gdk.CursorType.WATCH)
         self._dialog.get_window ().set_cursor (cursor)
 
@@ -622,61 +606,80 @@ class OpenSubtitles (GObject.Object, Peas.Activatable):
             subtitle_id = model.get_value (subtitle_iter, 3)
             subtitle_format = model.get_value (subtitle_iter, 1)
 
-            if not filename:
-                bpath = xdg.BaseDirectory.xdg_cache_home + sep
-                bpath += 'totem' + sep
+            bpath = _cache_subtitles_dir()
 
-                directory = Gio.file_new_for_path (bpath + 'subtitles' + sep)
-
-                if not directory.query_exists (None):
-                    if not path.exists (bpath):
-                        mkdir (bpath)
-                    if not path.exists (bpath + 'subtitles' + sep):
-                        mkdir (bpath + 'subtitles' + sep)
-                    # FIXME: We can't use this function until we depend on
-                    # GLib (PyGObject) 2.18
-                    # directory.make_directory_with_parents ()
-
-                subtitle_file = Gio.file_new_for_path (self._filename)
-                movie_name = subtitle_file.get_basename ().rpartition ('.')[0]
-
-                filename = directory.get_uri () + sep
-                filename += movie_name + '.' + subtitle_format
+            directory = Gio.file_new_for_path (bpath)
+            try:
+                directory.make_directory_with_parents (None)
+            except: # pylint: disable=bare-except
+                pass
 
             thread = DownloadThread (self._model, subtitle_id)
             thread.start ()
-            GObject.idle_add (self._save_subtitles, thread, filename)
+            GLib.idle_add (self._save_subtitles, thread, subtitle_format)
 
             self._progress.set_text (_(u'Downloading the subtitles…'))
-            GObject.timeout_add (350, self._progress_bar_increment, thread)
+            GLib.timeout_add (350, self._progress_bar_increment, thread)
         else:
             #warn user!
             pass
 
-    def _save_subtitles (self, download_thread, filename):
+    def _movie_dir (self):
+        directory = Gio.file_new_for_uri (self._filename)
+        parent = directory.get_parent()
+        return parent.get_path ()
+
+    def _save_subtitles (self, download_thread, extension):
         if not download_thread.done:
             return True
 
         subtitles = download_thread.get_subtitles ()
+        suburi = None
         if subtitles:
+            subtitle_file = Gio.file_new_for_uri (self._filename)
+            movie_name = subtitle_file.get_basename ().rpartition ('.')[0]
+
             # Delete all previous cached subtitle for this file
             for ext in SUBTITLES_EXT:
-                subtitle_file = Gio.file_new_for_path (filename[:-3] + ext)
-                if subtitle_file.query_exists (None):
-                    subtitle_file.delete (None)
+                # In the cache dir
+                old_subtitle_file = Gio.file_new_for_path (_cache_subtitles_dir() +
+                        sep + movie_name + '.' + ext)
+                try:
+                    old_subtitle_file.delete (None)
+                except: # pylint: disable=bare-except
+                    pass
 
-            subtitle_file = Gio.file_new_for_uri (filename)
-            suburi = subtitle_file.get_uri ()
+                # In the movie dir
+                old_subtitle_file = Gio.file_new_for_path (self._movie_dir() +
+                        sep + movie_name + '.' + ext)
+                try:
+                    old_subtitle_file.delete (None)
+                except: # pylint: disable=bare-except
+                    pass
 
-            sub_file = subtitle_file.replace ('', False)
-            sub_file.write (subtitles)
-            sub_file.close ()
+            flags = Gio.FileCreateFlags.REPLACE_DESTINATION
+            try:
+                subtitle_file = Gio.file_new_for_path (self._movie_dir() +
+                        sep + movie_name + '.' + extension)
+                suburi = subtitle_file.get_uri ()
+
+                sub_file = subtitle_file.replace ('', False, flags, None)
+                sub_file.write (subtitles, None)
+                sub_file.close (None)
+            except: # pylint: disable=bare-except
+                subtitle_file = Gio.file_new_for_path (_cache_subtitles_dir() +
+                        sep + movie_name + '.' + extension)
+                suburi = subtitle_file.get_uri ()
+
+                sub_file = subtitle_file.replace ('', False, flags, None)
+                sub_file.write (subtitles, None)
+                sub_file.close (None)
 
         self._dialog.get_window ().set_cursor (None)
         self._close_dialog ()
 
         if suburi:
-            self._totem.set_current_subtitle (suburi)
+            self._totem.set_current_subtitle (suburi) # pylint: disable=no-member
 
         return False
 
@@ -689,7 +692,7 @@ class OpenSubtitles (GObject.Object, Peas.Activatable):
         if message:
             self._progress.set_text (message)
         else:
-            self._progress.set_text ('')
+            self._progress.set_text (' ')
 
         self._progress.set_fraction (0.0)
         self._find_button.set_sensitive (True)
@@ -700,7 +703,7 @@ class OpenSubtitles (GObject.Object, Peas.Activatable):
     def _download_and_apply (self):
         self._apply_button.set_sensitive (False)
         self._find_button.set_sensitive (False)
-        self._action.set_sensitive (False)
+        self._action.set_enabled (False)
         self._tree_view.set_sensitive (False)
         self._save_selected_subtitle ()
 
@@ -712,7 +715,7 @@ class OpenSubtitles (GObject.Object, Peas.Activatable):
 
     # Callbacks
 
-    def __on_window__key_press_event (self, _widget, event):
+    def __on_window__key_press_event (self, _, event):
         if event.keyval == Gdk.KEY_Escape:
             self._close_dialog ()
             return True
@@ -724,13 +727,16 @@ class OpenSubtitles (GObject.Object, Peas.Activatable):
         else:
             self._apply_button.set_sensitive (False)
 
-    def __on_treeview__row_activate (self, _tree_path, _column, _data):
+    def __on_treeview__row_activate (self,
+                                     _tree_path, # pylint: disable=W0613
+                                     _column, # pylint: disable=W0613
+                                     _data): # pylint: disable=W0613
         self._download_and_apply ()
 
-    def __on_totem__file_opened (self, _totem, new_mrl):
+    def __on_totem__file_opened (self, _, new_mrl):
         # Check if allows subtitles
         if self._check_allowed_scheme () and not self._check_is_audio ():
-            self._action.set_sensitive (True)
+            self._action.set_enabled (True)
             if self._dialog:
                 self._find_button.set_sensitive (True)
                 # Check we're not re-opening the same file; if we are, don't
@@ -739,19 +745,17 @@ class OpenSubtitles (GObject.Object, Peas.Activatable):
                 if self._filename != new_mrl:
                     self._filename = new_mrl
                     self._list_store.clear ()
-                    self._tree_view.set_headers_visible (False)
                     self._apply_button.set_sensitive (False)
         else:
-            self._action.set_sensitive (False)
+            self._action.set_enabled (False)
             if self._dialog and self._dialog.is_active ():
                 self._filename = None
                 self._list_store.clear ()
-                self._tree_view.set_headers_visible (False)
                 self._apply_button.set_sensitive (False)
                 self._find_button.set_sensitive (False)
 
-    def __on_totem__file_closed (self, _totem):
-        self._action.set_sensitive (False)
+    def __on_totem__file_closed (self, _):
+        self._action.set_enabled (False)
         if self._dialog:
             self._apply_button.set_sensitive (False)
             self._find_button.set_sensitive (False)
@@ -762,17 +766,16 @@ class OpenSubtitles (GObject.Object, Peas.Activatable):
         self._model.lang = LANGUAGES[combo_model.get_value (combo_iter, 1)]
         self._settings.set_string ('language', self._model.lang)
 
-    def __on_close_clicked (self, _data):
+    def __on_close_clicked (self, _):
         self._close_dialog ()
 
-    def __on_apply_clicked (self, _data):
+    def __on_apply_clicked (self, _):
         self._download_and_apply ()
 
-    def __on_find_clicked (self, _data):
+    def __on_find_clicked (self, _):
         self._apply_button.set_sensitive (False)
         self._find_button.set_sensitive (False)
-        self._filename = self._totem.get_current_mrl ()
+        self._filename = self._totem.get_current_mrl () # pylint: disable=no-member
         (movie_hash, movie_size) = hash_file (self._filename)
 
         self._get_results (movie_hash, movie_size)
-
